/****************************************************************************
 *
 *                                 M U E S L I   v 1.0
 *
 *
 *     Copyright 2016 IMDEA Materials Institute, Getafe, Madrid, Spain
 *     Contact: muesli.materials@imdea.org
 *     Author: Ignacio Romero (ignacio.romero@imdea.org)
 *
 *     This file is part of MUESLI.
 *
 *     MUESLI is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     MUESLI is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with MUESLI.  If not, see <http://www.gnu.org/licenses/>.
 *
*****************************************************************************/



#pragma once
#ifndef _muesli_svk_
#define _muesli_svk_


#include "finitestrain.h"


namespace muesli
{


    class svkMaterial : public muesli::finiteStrainMaterial
    {

    public:
                                        svkMaterial(const std::string& name,
                                                    const materialProperties& cl);
                                        svkMaterial();
        virtual                         ~svkMaterial(){}

        virtual double                  characteristicStiffness() const;
        virtual muesli::finiteStrainMP* createMaterialPoint() const;
        void                            print(std::ostream &of=std::cout) const;
        virtual void                    setRandom();
        virtual bool                    test(std::ostream &of=std::cout);


    private:
        double                          E, nu, lambda, mu;

        friend class svkMP;
    };




    class svkMP : public muesli::finiteStrainMP
    {

    public:
                                        svkMP(const svkMaterial &m);
        virtual                         ~svkMP(){}

        // energy
        virtual double                  dissipatedEnergy() const;
        virtual double                  storedEnergy() const;
        virtual double                  energyDissipationPotential() const;


        // stress
        virtual void                    CauchyStress(istensor &sigma) const;
        virtual void                    secondPiolaKirchhoffStress(istensor &S) const;


        // material elasticities
        virtual void                    convectedTangent(itensor4& c) const;
        virtual void                    materialTangent(itensor4& c) const;
        virtual void                    materialTangentMatrix(double C[6][6]) const;
        virtual void                    spatialTangent(itensor4& c) const;


        // tangent operations
        virtual void                    contractWithMaterialTangent(const ivector &V, const ivector &W,itensor &T) const;
        virtual void                    contractWithSpatialTangent(const ivector &v1, const ivector &v2,itensor &T) const;
        virtual void                    materialTangentTimesSymmetricTensor(const istensor& M,istensor& CM) const;
        virtual void                    contractWithDeviatoricTangent(const ivector &v1, const ivector& v2,itensor &T) const {}
        virtual void                    contractWithMixedTangent(istensor& CM) const {}
        virtual double                  volumetricStiffness() const {return 0.0;}


        // for 1-dimensional problems
        virtual double                  stress(const double stretch) const;
        virtual double                  stiffness(const double stretch) const;
        virtual double                  storedEnergy(const double stretch) const;


        // interfacing with other codes
        void                            setConvergedState(const double time, const itensor& Fn);


        virtual void                    setRandom(){};
        virtual double                  plasticSlip() const;
        
    private:
        
        svkMaterial const               *mat;
    };
}


#endif
