/****************************************************************************
 *
 *                                 M U E S L I   v 1.0
 *
 *
 *     Copyright 2016 IMDEA Materials Institute, Getafe, Madrid, Spain
 *     Contact: muesli.materials@imdea.org
 *     Author: Ignacio Romero (ignacio.romero@imdea.org)
 *
 *     This file is part of MUESLI.
 *
 *     MUESLI is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     MUESLI is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with MUESLI.  If not, see <http://www.gnu.org/licenses/>.
 *
*****************************************************************************/




#pragma once
#ifndef __muesli_smallthermo_h
#define __muesli_smallthermo_h

#include "muesli/material.h"


namespace muesli
{
    class smallStrainMaterial;
    class smallStrainMP;
    class smallThermomechanicalMP;

    class smallThermomechanicalMaterial : public muesli::material
    {
	public:
                                smallThermomechanicalMaterial(){};
                                smallThermomechanicalMaterial(const std::string& name,
                                                              const materialProperties& cl);

        virtual                 ~smallThermomechanicalMaterial();

        virtual bool            check() const = 0;
        virtual smallThermomechanicalMP*	createMaterialPoint() const = 0;
        virtual double          density() const = 0;
        virtual double          getProperty(const propertyName p) const = 0;
        virtual void			print(std::ostream &of=std::cout) const = 0;
        virtual void            setRandom() = 0;
        virtual bool            test() = 0;
        virtual double          waveVelocity() const = 0;


    private:
        smallStrainMaterial     *theSSMaterial;
        double                  alpha;
        double                  tref;
        double                  conductivity;

        friend class smallThermomechanicalMP;
    };




    class smallThermomechanicalMP : public muesli::materialPoint
    {

    public:
                                smallThermomechanicalMP(const smallThermomechanicalMaterial &m);
        virtual                 ~smallThermomechanicalMP();
        virtual void            setRandom() = 0;
        bool                    testImplementation() const;


        // energies
        virtual double          freeEnergy() const;
        virtual double          deviatoricEnergy() const = 0;
        virtual double          dissipatedEnergy() const = 0;
        virtual double          energyDissipationPotential() const = 0;
        virtual double          volumetricEnergy() const = 0;


        // gradients
        virtual double          entropy() const;
        virtual ivector         heatflux() const;
        virtual double          pressure() const;
        virtual void            stress(istensor& sigma) const;
        virtual void            deviatoricStress(istensor& s) const;
        virtual double          plasticSlip() const;


        // tangents
        virtual void			contractWithTangent(const ivector &v1,
                                                    const ivector &v2,
                                                    itensor &T) const=0;
        virtual void			contractWithDeviatoricTangent(const ivector &v1,
                                                              const ivector &v2,
                                                              itensor &T) const = 0;
        virtual double          volumetricStiffness() const = 0;


        // uniaxial response
        virtual double			uniaxialStiffness() const = 0;
        virtual double          uniaxialStress() const = 0;


        // bookkeeping
        virtual void            updateCurrentState(const double t, const double strain11, const double temp, const double gradt);
        virtual void            updateCurrentState(const double t, const istensor& strain, const double temp, const ivector& gradt);
        virtual void            commitCurrentState();
        virtual void            resetCurrentState();


        double                  density() const;
        const smallThermomechanicalMaterial& parentMaterial() const;


    private:
        double                                  temp_n, temp_c;
        ivector                                 gradt_n, gradt_c;
        smallStrainMP*                          theSSMP;
        const smallThermomechanicalMaterial&	theThermoMechMaterial;
    };

    inline double smallThermomechanicalMP :: density() const  {return theThermoMechMaterial.density();}
    inline const smallThermomechanicalMaterial& smallThermomechanicalMP :: parentMaterial() const {return theThermoMechMaterial;}
}

#endif
